// Boost.Process
//
// Copyright (c) 2006 Julio M. Merino Vidal
// Copyright (c) 2009 Ilya Sokolov
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt.)

#ifndef BOOST_PROCESS_COMMAND_LINE_HPP_150112
#define BOOST_PROCESS_COMMAND_LINE_HPP_150112

#include "boost/process/config.hpp"
#include <string>
#include <vector>

#ifdef BOOST_MSVC
#pragma warning(push)
// 4251 * needs to have dll-interface to be used by clients of *
#pragma warning(disable: 4251)
#endif

namespace boost { namespace processes {

struct no_default_arg0_t {};
const no_default_arg0_t no_default_arg0 = {};

class BOOST_PROCESS_DECL command_line
{
public:
    typedef std::vector<std::string> arguments_type;

    command_line(const std::string& executable)
    : executable_(executable)
    {
        push_back_default_arg0();
    }

    command_line(const std::string& executable, no_default_arg0_t)
    : executable_(executable)
    {}

    command_line(const std::string& executable,
                 const std::string& arg1)
    : executable_(executable)
    {
        arguments_.reserve(2);
        push_back_default_arg0();
        push_back(arg1);
    }

    command_line(const std::string& executable,
                 const std::string& arg1,
                 const std::string& arg2)
    : executable_(executable)
    {
        arguments_.reserve(3);
        push_back_default_arg0();
        push_back(arg1);
        push_back(arg2);
    }

    command_line(const std::string& executable,
                 const std::string& arg1,
                 const std::string& arg2,
                 const std::string& arg3)
    : executable_(executable)
    {
        arguments_.reserve(4);
        push_back_default_arg0();
        push_back(arg1);
        push_back(arg2);
        push_back(arg3);
    }

    command_line(const std::string& executable,
                 const std::string& arg1,
                 const std::string& arg2,
                 const std::string& arg3,
                 const std::string& arg4)
    : executable_(executable)
    {
        arguments_.reserve(5);
        push_back_default_arg0();
        push_back(arg1);
        push_back(arg2);
        push_back(arg3);
        push_back(arg4);
    }

    command_line(const std::string& executable,
                 const std::string& arg1,
                 const std::string& arg2,
                 const std::string& arg3,
                 const std::string& arg4,
                 const std::string& arg5)
    : executable_(executable)
    {
        arguments_.reserve(6);
        push_back_default_arg0();
        push_back(arg1);
        push_back(arg2);
        push_back(arg3);
        push_back(arg4);
        push_back(arg5);
    }

    command_line& push_back(const std::string& argument)
    {
        arguments_.push_back(argument);
        return *this;
    }

    const arguments_type& arguments() const
    {
        return arguments_;
    }

    const std::string& executable() const
    {
        return executable_;
    }

private:
    void push_back_default_arg0()
    {
        std::string::size_type pos

#if defined(BOOST_POSIX_API)
            = executable_.rfind('/');
#elif defined(BOOST_WINDOWS_API)
            = executable_.rfind('\\');
        if (pos == std::string::npos)
            pos = executable_.rfind('/');
#endif

        arguments_.push_back(pos == std::string::npos
                                ? executable_
                                : executable_.substr(pos + 1));
    }

    std::string executable_;
    arguments_type arguments_;
};

}} // namespace boost::processes

#ifdef BOOST_MSVC
#pragma warning(pop)
#endif

#endif
